
@font-face {
    font-family: 'Overpass';
    src: url('overpass_regular-web.eot');
    src: local('Overpass'),
	 url('overpass_regular-web.eot?#iefix') format('eot'),
         url('overpass_regular-web.woff') format('woff'),
         url('overpass_regular-web.ttf') format('truetype'),
         url('overpass_regular-web.svg#webfontLTZe4IYH') format('svg');
    font-weight: normal;
    font-style: normal;

}

@font-face {
    font-family: 'Overpass';
    src: url('overpass_bold-web.eot');
    src: local('Overpass Bold'), local('Overpass-Bold'),
	 url('overpass_bold-web.eot?#iefix') format('eot'),
         url('overpass_bold-web.woff') format('woff'),
         url('overpass_bold-web.ttf') format('truetype'),
         url('overpass_bold-web.svg#webfontzAU82Ltw') format('svg');
    font-weight: bold;
    font-style: normal;

}

